@@ -2,7 +2,7 @@
 Author:
 Thomas Mayer <thomas@residuum.org>
 
-Copyright (c) 2011-2015 Thomas Mayer
+Copyright (c) 2011-2022 Thomas Mayer
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,15 +35,25 @@ static t_class *json_decode_class;
 
 struct _json_decode {
 	t_object x_ob;
-	t_outlet *done_outlet;
+	t_outlet *data_outlet;
+	t_outlet *error_outlet;
 };
 
+/* constructor */
+static void *json_decode_new(const t_symbol *sel, const int argc, const t_atom *argv);
+
+/** Functions called via Pd messages **/
+/* string input */
+static void json_decode_string(t_json_decode *x, const t_symbol *data);
+/* list input */
+static void json_decode_list(t_json_decode *x, const t_symbol *sel, const int argc, t_atom *argv);
+
 /* outputs json object at outlets */
 static void jdec_output_object(json_object *jobj, t_outlet *data_outlet, t_outlet *done_outlet);
 /* outputs json array at outlets */
-static void jdec_output_array(json_object *jobj, t_outlet *data_outlet, t_outlet *done_outlet);
+static void jdec_output_array(json_object *jobj, t_outlet *data_outlet, t_outlet *done_outlet, t_outlet *error_outlet);
 /* output json data */
-static void jdec_output(json_object *jobj, t_outlet *data_outlet, t_outlet *done_outlet);
+static void jdec_output(json_object *jobj, t_outlet *data_outlet, t_outlet *done_outlet, t_outlet *error_outlet);
 /* checks for valid json string and converts to json */
 static void jdec_output_string(const char *json_string, t_json_decode *jdec);
 
@@ -68,20 +78,20 @@ static void jdec_output_object(json_object *const jobj, t_outlet *const data_out
 				case json_type_int:
 					SETFLOAT(&out_data[1], json_object_get_int(val));
 					break;
-				case json_type_string: 
+				case json_type_string:
 					SETSYMBOL(&out_data[1], gensym(json_object_get_string(val)));
 					break;
 				case json_type_object:
-					SETSYMBOL(&out_data[1], gensym(json_object_get_string(val)));
+					SETSYMBOL(&out_data[1], gensym(json_object_to_json_string_ext(val, JSON_C_TO_STRING_PLAIN)));
 					break;
 				case json_type_array:
-					SETSYMBOL(&out_data[1], gensym(json_object_get_string(val)));
+					SETSYMBOL(&out_data[1], gensym(json_object_to_json_string_ext(val, JSON_C_TO_STRING_PLAIN)));
 					break;
 				case json_type_null:
 					SETSYMBOL(&out_data[1], gensym(""));
 					break;
 				default:
-					MYERROR("What other JSON type?");
+					pd_error(0, "What other JSON type?");
 					break;
 			}
 		}
@@ -90,17 +100,19 @@ static void jdec_output_object(json_object *const jobj, t_outlet *const data_out
 	outlet_bang(done_outlet);
 }
 
-static void jdec_output_array(json_object *jobj, t_outlet *const data_outlet, t_outlet *const done_outlet) {
+static void jdec_output_array(json_object *jobj, t_outlet *const data_outlet, t_outlet *const done_outlet, t_outlet *const error_outlet) {
 	const int array_len = json_object_array_length(jobj);
 	for (int i = 0; i < array_len; i++) {
 		json_object *array_member = json_object_array_get_idx(jobj, i);
 		if (array_member != NULL) {
-			jdec_output(array_member, data_outlet, done_outlet);
+			jdec_output(array_member, data_outlet, done_outlet, error_outlet);
+		} else {
+			outlet_bang(error_outlet);
 		}
 	}
 }
 
-static void jdec_output(json_object *const jobj, t_outlet *const data_outlet, t_outlet *const done_outlet) {
+static void jdec_output(json_object *const jobj, t_outlet *const data_outlet, t_outlet *const done_outlet, t_outlet *const error_outlet) {
 	t_atom out_data[2];
 	t_float out_float;
 	const enum json_type outer_type = json_object_get_type(jobj);
@@ -135,11 +147,11 @@ static void jdec_output(json_object *const jobj, t_outlet *const data_outlet, t_
 		case json_type_object:
 			jdec_output_object(jobj, data_outlet, done_outlet);
 			break;
-		case json_type_array: 
-			jdec_output_array(jobj, data_outlet, done_outlet);
+		case json_type_array:
+			jdec_output_array(jobj, data_outlet, done_outlet, error_outlet);
 			break;
 		default:
-			MYERROR("What other JSON type?");
+			pd_error(0, "What other JSON type?");
 			break;
 	}
 }
@@ -148,10 +160,11 @@ static void jdec_output_string(const char *const json_string, t_json_decode *con
 	json_object *const jobj = json_tokener_parse(json_string);
 
 	if (jobj != NULL) {
-		jdec_output(jobj, jdec->x_ob.ob_outlet, jdec->done_outlet);
+		jdec_output(jobj, jdec->data_outlet, jdec->x_ob.ob_outlet, jdec->error_outlet);
 		json_object_put(jobj);
 	} else {
 		pd_error(jdec, "Not a JSON object.");
+		outlet_bang(jdec->error_outlet);
 	}
 }
 
@@ -170,14 +183,15 @@ void *json_decode_new(const t_symbol *const sel, const int argc, const t_atom *c
 	(void) argc;
 	(void) argv;
 
-	outlet_new(&jdec->x_ob, NULL);
-	jdec->done_outlet = outlet_new(&jdec->x_ob, &s_bang);
+	outlet_new(&jdec->x_ob, &s_bang);
+	jdec->data_outlet = outlet_new(&jdec->x_ob, NULL);
+	jdec->error_outlet = outlet_new(&jdec->x_ob, &s_bang);
 	purest_json_lib_info("json-decode");
 	return (void *)jdec;
 }
 
 void json_decode_string(t_json_decode *const jdec, const t_symbol *const data) {
-	char *const original_string = data->s_name;
+	const char *const original_string = data->s_name;
 
 	if (original_string && strlen(original_string)) {
 		size_t memsize = 0;
