@@ -2,7 +2,7 @@
 Author:
 Thomas Mayer <thomas@residuum.org>
 
-Copyright (c) 2011-2015 Thomas Mayer
+Copyright (c) 2011-2022 Thomas Mayer
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,21 +26,18 @@ THE SOFTWARE.
 
 #include "m_pd.h"
 
-
 #define REQUEST_TYPE_LEN 8
 
 #ifdef _WIN32
 	#define APIEXPORT __declspec(dllexport)
 	#define APICALL __cdecl
-	#define MYERROR(...) post(__VA_ARGS__)
-	#define NEEDS_CERT_PATH 1
-#else 
-	#define APIEXPORT
-	#define APICALL
-	#define MYERROR(...) error(__VA_ARGS__)
-	#ifdef __APPLE__
-		#define NEEDS_CERT_PATH 1
+#else
+	#if defined(__GNUC__) && __GNUC__>=4
+		#define APIEXPORT extern __attribute__ ((visibility("default")))
+	#else
+		#define APIEXPORT
 	#endif
+	#define APICALL
 #endif
 
 static void purest_json_lib_info(char *name) {
@@ -49,7 +46,7 @@ static void purest_json_lib_info(char *name) {
 
 #define MYASSERT(cond, ...) do { \
     if (!(cond)) {               \
-        MYERROR(__VA_ARGS__);    \
+        pd_error(0, __VA_ARGS__);    \
     }                            \
 } while(0);
 
@@ -68,5 +65,5 @@ APIEXPORT void APICALL setup_json0x2ddecode(void);
 /* [urlparams] */
 APIEXPORT void APICALL urlparams_setup(void);
 
-/* general */ 
+/* general */
 APIEXPORT void APICALL purest_json_setup(void);
